<?php

/**
 * @file
 * Functions for communicating with the GoCardless partner: Seamless-CMS.co.uk.
 */
class GoCardlessPartner {

  public function __construct() {
    $this->settings = $this->getSettings();
  }

  public static function getSettings() {

    $settings = commerce_gc_client_load_settings() + commerce_gc_client_default_settings();
    $sandbox = $settings['connect']['sandbox'];
    $sandbox ? $ext = '_sandbox' : $ext = '_live';
    $settings['partner_user'] = variable_get('commerce_gc_client_user_name' . $ext);
    $settings['partner_pass'] = variable_get('commerce_gc_client_user_pass' . $ext);
    $settings['org_id'] = variable_get('commerce_gc_client_org_id' . $ext);
    $settings['webhook'] = $settings['connect']['webhook' . $ext];
    $settings['partner_url'] = variable_get('commerce_gc_client_partner_url');
    return $settings;
  }

  /**
   * Handles Sevices module session authentication with GC Partner site.
   */
  protected function authenticate() {

    if (isset($_SESSION['gc_client_cookie_created']) && $_SESSION['gc_client_cookie_created'] < REQUEST_TIME - 1800) {
      unset($_SESSION['gc_client_cookie']);
      unset($_SESSION['gc_client_cookie_created']);
    }

    if (!isset($_SESSION['gc_client_cookie_created'])) {
      $sandbox = $this->settings['connect']['sandbox'] ? $ext = '_sandbox' : $ext = '_live';

      // Login first.
      $data = array(
        'username' => $this->settings['partner_user'],
        'password' => $this->settings['partner_pass'],
      );
      $data = drupal_json_encode($data);
      $url = $this->settings['partner_url'] . '/gc_connect/user/login';
      $options = array(
        'headers' => array(
          'Content-Type' => 'application/json',
        ),
        'method' => 'POST',
        'data' => $data,
      );
      $result = drupal_http_request($url, $options);
      $result_data = json_decode($result->data);
      if ($result->code == 200) {

        // Get X-CRSF token, and save cookie and token.
        $_SESSION['gc_client_cookie'] = $result_data->session_name . '=' . $result_data->sessid;
        $_SESSION['gc_client_cookie_created'] = REQUEST_TIME;
        $xcrf_url = $this->settings['partner_url'] . '/services/session/token';
        $xcrf_options = array(
          'method' => 'GET',
          'headers' => array(
            'Cookie' => $_SESSION['gc_client_cookie'],
          ),
        );
        $xcrf_result = drupal_http_request($xcrf_url, $xcrf_options);
        $token = $xcrf_result->data;
        $_SESSION['gc_client_token'] = $token;
      }
      return $result->code;
    }
    else {
      // Already logged in.
      return $result = 200;
    }
  }

  /**
   * Handles API posts to GC Partner site, and optionally logs results.
   */
  public function api($params) {

    $auth = $this->authenticate();
    if (empty($params)) {
      return $auth;
    }
    if ($auth == 200) {
      $result = $this->post($params);
      if (isset($result->error)) {
        $message = t('Error calling GoCardless API, code @code (@error)', array('@code' => $result->code, '@error' => $result->error,
          ));
        drupal_set_message($message, 'error');
        if ($this->settings['log']['log_api']) {
          watchdog("commerce_gc_client", '<pre>' . $message . '<br />' . print_r($result, TRUE) . '</pre>', NULL, WATCHDOG_ERROR);
        }
        $result->response = new stdClass();
        $result->response->status_code = $result->code;
        return $result;
      }
      else {
        if ($this->settings['log']['log_api']) {
          watchdog("commerce_gc_client", '<pre>GoCardless API response: <br />' . print_r($result, TRUE) . '</pre>');
        }
        return $result;
      }
    }
    else {
      drupal_set_message(t('Error @code connecting with partner site', array('@code' => $auth)), 'error');
      if (variable_get('commerce_gc_client_debug_api', FALSE)) {
        watchdog("commerce_gc_client", '<pre>' . print_r($auth, TRUE) . '</pre>', NULL, WATCHDOG_ERROR);
      }
      return $auth;
    }
  }

  /**
   * Handles HTTP requests to GC Partner site.
   */
  private function post($data) {
    variable_set('date_format_gocardless', 'Y-m-d');
    $options['headers'] = array(
      'Content-Type' => 'application/json',
      'Cookie' => $_SESSION['gc_client_cookie'],
      'X-CSRF-Token' => $_SESSION['gc_client_token'],
    );
    $this->settings['connect']['sandbox'] ? $data['environment'] = 'SANDBOX' : $data['environment'] = 'LIVE';
    $this->settings['connect']['sandbox'] ? $ext = '_sandbox' : $ext = '_live';
    $data = drupal_json_encode($data);
    $options['data'] = $data;
    $options['method'] = 'POST';
    $url = $this->settings['partner_url'] . '/gc/client/' . $this->settings['org_id'];
    $response = drupal_http_request($url, $options);

    if (isset($response->error)) {

      return $response;

    }
    else {
      $data = json_decode($response->data);
      return $data;
    }
  }
}

